@@ -46,8 +46,5 @@
 // ~~~~~~~~~~~~~~~~~~~~
 
 // char.diff
-double          character_to_numeric(char c);
-void            Normalise_single_character(double *vector, int count);
-void            R_distance(double *x, int *nr, int *nc, double *d, int *diag, int *method);
-SEXP            C_char_diff(SEXP x, SEXP smethod, SEXP attrs);
+SEXP            C_bitwisedist(SEXP x, SEXP smethod, SEXP stranslate, SEXP order, SEXP attrs);
 
